    document.addEventListener("keydown", function(event) {
        const searchInput = document.getElementById('tile-search'); 
        
        if (event.code === "Space") {
            isSpacePressed = true;
        } else if (event.code === "KeyA" && isSpacePressed) {
            toggleSorting();
        } else if (event.code === "KeyR" && isSpacePressed) {
            toggleRamSorting();
        } else if (event.code === "KeyC" && isSpacePressed) {
            toggleCpuSorting();
        } else if (event.code === "KeyV" && isSpacePressed) {
            toggleView();
        } else if (event.code === "KeyM" && isSpacePressed) {
            settingsButton.click();
        } else if (event.code === "Slash") { 
            event.preventDefault();
            if (searchInput) {
                searchInput.focus(); 
            }
        } else if (event.code === "Escape") { 
            if (searchInput && document.activeElement === searchInput) {
                searchInput.blur(); 
            }
        }
    });


    document.addEventListener("keyup", function(event) {
        if (event.code === "Space") {
            isSpacePressed = false;
        }
    });

    window.addEventListener('resize', () => {
        if (window.innerWidth >= 1024) {
            settingsPanel.classList.add('desktop');
        } else {
            settingsPanel.classList.remove('desktop');
        }
    });

    sortToggleSpan.addEventListener('click', () => {
        sortToggleCheckbox.checked = !sortToggleCheckbox.checked;
        storedSortingEnabled = sortToggleCheckbox.checked;
        toggleSorting();
    });

    cornerToggleSpan.addEventListener('click', () => {
        cornerToggleCheckbox.checked = !cornerToggleCheckbox.checked;
        storedStraightEdges = cornerToggleCheckbox.checked;
        applyCornerStyle();
    });

    document.getElementById('grid-view-button').addEventListener('click', switchToGridView);
    document.getElementById('list-view-button').addEventListener('click', switchToListView);

    document.getElementById('master-tile-color').addEventListener('input', (e) => {
        const newColor = e.target.value;
        masterTileColor = newColor;
        localStorage.setItem('masterTileColor', newColor);
        applyTileColors();
    });

    const tileOpacityInput = document.getElementById('tile-opacity');
    tileOpacityInput.value = storedTileOpacity;
    tileOpacityInput.addEventListener('input', (e) => {
        const opacity = e.target.value;
        document.documentElement.style.setProperty('--tile-background-opacity', opacity);
        localStorage.setItem('tileOpacity', opacity);
    });

    const avatarInput = document.getElementById('avatar-upload');
    avatarInput.addEventListener('change', (e) => {
        const file = e.target.files[0];
        if (file) {
            const reader = new FileReader();
            reader.onload = function(event) {
                const avatarUrl = event.target.result;
                avatarElement.src = avatarUrl;
                localStorage.setItem('userAvatar', avatarUrl);
            };
            reader.readAsDataURL(file);
        }
    });

    const backgroundInput = document.getElementById('background-upload');
    backgroundInput.addEventListener('change', (e) => {
        const file = e.target.files[0];
        if (file) {
            // Check if the file size exceeds 1.4MB
            if (file.size > 1.4 * 1024 * 1024) {
                const reader = new FileReader();
                reader.onload = function(event) {
                    const imageUrl = event.target.result;
                    
                    // Create an image element
                    const img = new Image();
                    img.src = imageUrl;
                    
                    img.onload = function() {
                        // Create a canvas to resize the image
                        const canvas = document.createElement('canvas');
                        const maxWidth = 1920; // You can adjust this value
                        const maxHeight = 1080; // You can adjust this value
                        let width = img.width;
                        let height = img.height;
    
                        // Maintain aspect ratio
                        if (width > height) {
                            if (width > maxWidth) {
                                height = Math.round((height * maxWidth) / width);
                                width = maxWidth;
                            }
                        } else {
                            if (height > maxHeight) {
                                width = Math.round((width * maxHeight) / height);
                                height = maxHeight;
                            }
                        }
    
                        // Set canvas dimensions
                        canvas.width = width;
                        canvas.height = height;
    
                        const ctx = canvas.getContext('2d');
                        ctx.drawImage(img, 0, 0, width, height);
    
                        // Compress the image
                        const compressedImageUrl = canvas.toDataURL('image/jpeg', 0.7); // Compression quality (0.7 is 70%)
    
                        // Apply the compressed image as background
                        document.body.style.setProperty('--background-image', `url("${compressedImageUrl}")`);
                        localStorage.setItem('backgroundImage', compressedImageUrl);
                    };
                };
                reader.readAsDataURL(file);
            } else {
                // No resizing needed, store the image as is
                const reader = new FileReader();
                reader.onload = function(event) {
                    const imageUrl = event.target.result;
                    document.body.style.setProperty('--background-image', `url("${imageUrl}")`);
                    localStorage.setItem('backgroundImage', imageUrl);
                };
                reader.readAsDataURL(file);
            }
        }
    });
    

    // Function to toggle the settings panel
    settingsButton.addEventListener('click', () => {
        if (settingsPanel.classList.contains('visible')) {
            // Hide the settings panel
            settingsPanel.classList.remove('visible');
            settingsPanel.classList.add('hidden');
        } else {
            // Show the settings panel
            settingsPanel.classList.add('visible');
            settingsPanel.classList.remove('hidden');
        }
    });

    searchInput.addEventListener('input', function () {
        const searchTerm = searchInput.value.toLowerCase();
        tileMap.forEach((tile) => {
            const hostname = tile.hostname.toLowerCase();
            if (hostname.includes(searchTerm)) {
                tile.element.style.display = 'block';
            } else {
                tile.element.style.display = 'none';
            }
        });
    });

    document.querySelectorAll('.category-title').forEach(title => {
        title.addEventListener('click', () => {
            const contentId = title.getAttribute('data-target');
            document.getElementById(contentId).style.display =
                document.getElementById(contentId).style.display === 'none' ? 'block' : 'none';
        });
    });

    const backgroundOpacityInput = document.getElementById('background-opacity');
    backgroundOpacityInput.addEventListener('input', (e) => {
        const opacity = e.target.value;
        document.body.style.setProperty('--background-opacity', opacity);
        localStorage.setItem('backgroundOpacity', opacity);
    });

    document.getElementById('save-settings').addEventListener('click', () => {
        const username = document.getElementById('username-input').value;
        const pollingFrequency = document.getElementById('polling-frequency').value;
        const tileOpacity = document.getElementById('tile-opacity').value;
        const masterColor = document.getElementById('master-tile-color').value;

        masterTileColor = masterColor;

        const isCornerToggle = document.getElementById('corner-style').value;
        const isSortToggle = document.getElementById('sort-tiles').value;
        const cornerToggle = document.getElementById('corner-style').checked;
        const sortingToggle = document.getElementById('sort-tiles').checked;

        localStorage.setItem('username', username);
        localStorage.setItem('pollingFrequency', pollingFrequency);
        localStorage.setItem('tileOpacity', tileOpacity);
        localStorage.setItem('masterTileColor', masterColor);
        localStorage.setItem('straightEdges', JSON.stringify(cornerToggle));
        localStorage.setItem('sortTilesAlphabetically', JSON.stringify(sortingToggle));

        document.querySelector('.username').textContent = username;
        document.documentElement.style.setProperty('--tile-background-opacity', tileOpacity);
        applyTileColors();

        storedSortingEnabled = sortingToggle;
        storedStraightEdges = cornerToggle;

        if (pollingFrequency >= 1) {
            applyPollingFrequency(pollingFrequency);
        }

        if (settingsPanel.classList.contains('visible')) {
            settingsPanel.classList.remove('visible');
            settingsPanel.classList.add('hidden');
        } else {
            settingsPanel.classList.remove('hidden');
            settingsPanel.classList.add('visible');
        }
    });

    document.addEventListener('click', function(event) {
        const isClickInside = settingsPanel.contains(event.target);
        const isClickOnSettingsButton = event.target.id === 'settings-button';

        if (!isClickInside && !isClickOnSettingsButton) {
            closeSettingsPanel();
        }
    });

    document.getElementById('font-color').addEventListener('input', (e) => {
        const newFontColor = e.target.value;
        storedFontColor = newFontColor;
        document.documentElement.style.setProperty('--font-color', newFontColor);
        localStorage.setItem('fontColor', newFontColor);

        // Apply font color to text and avatar border
        applyFontColor(newFontColor);

        // Change the Save button text
        document.getElementById('save-settings').textContent = "Saving Changes...";

        // Add timeout to revert Save button text back
        setTimeout(() => {
            document.getElementById('save-settings').textContent = "Save Settings";
        }, 2000);
    });

    document.getElementById('avatar-button').addEventListener('click', function() {
        document.getElementById('avatar-upload').click();
    });

    document.getElementById('background-button').addEventListener('click', function() {
        document.getElementById('background-upload').click();
    });

