let tileMap = new Map();
let tileIndex = 0;
    
    function playSoundFromUrl(url) {
        const audio = new Audio(url);
        audio.play().catch(error => {
            console.error("Error playing sound:", error);
        });
    }

    function sortTilesByRamUsage() {
        const tileContainer = document.querySelector('.tile-container');
        const sortedTiles = Array.from(tileMap.values()).sort((a, b) => b.ramUsage - a.ramUsage);
        sortedTiles.forEach(tile => tileContainer.appendChild(tile.element));
    }

    function sortTilesByCpuUsage() {
        const tileContainer = document.querySelector('.tile-container');
        const sortedTiles = Array.from(tileMap.values()).sort((a, b) => b.cpuUsage - a.cpuUsage);
        sortedTiles.forEach(tile => tileContainer.appendChild(tile.element));
    }

    function toggleRamSorting() {
        isRamSortingEnabled = !isRamSortingEnabled;
        isCpuSortingEnabled = false;

        if (isRamSortingEnabled) {
            sortTilesByRamUsage();
            new Toast({
                message: 'Tiles have been sorted by RAM usage (highest to lowest).',
                backgroundColor: '#28a745',
                autoDismiss: true
            });
        } else {
            restoreOriginalTileOrder();
            new Toast({
                message: 'Tiles have been returned to default sorting.',
                backgroundColor: '#28a745',
                autoDismiss: true
            });
        }
    }

    function toggleCpuSorting() {
        isCpuSortingEnabled = !isCpuSortingEnabled;
        isRamSortingEnabled = false;

        if (isCpuSortingEnabled) {
            sortTilesByCpuUsage();
            new Toast({
                message: 'Tiles have been sorted by CPU usage (highest to lowest).',
                backgroundColor: '#28a745',
                autoDismiss: true
            });
        } else {
            restoreOriginalTileOrder();
            new Toast({
                message: 'Tiles have been returned to default sorting.',
                backgroundColor: '#28a745',
                autoDismiss: true
            });
        }
    }

    function applyPollingFrequency(newFrequency) {
        pollingFrequency = newFrequency * 1000;
        localStorage.setItem('pollingFrequency', pollingFrequency);
    }

    function applyCornerStyle() {
        const tiles = document.querySelectorAll('.tile');
        tiles.forEach(tile => {
            if (storedStraightEdges) {
                tile.style.borderRadius = '0';
            } else {
                tile.style.borderRadius = '5px';
            }
        });
    }

    function applyFontColor(color) {
        const textElements = document.querySelectorAll('.tile-text, .username, .tile-data, body');
        textElements.forEach(element => {
            element.style.color = color;
        });

        // Apply font color to avatar border
        const avatarElement = document.querySelector('.user-avatar');
        avatarElement.style.borderColor = color;
    }

    function toggleSorting() {
        storedSortingEnabled = !storedSortingEnabled;
        isRamSortingEnabled = false;
        isCpuSortingEnabled = false;

        if (storedSortingEnabled) {
            sortTilesAlphabetically();
            new Toast({
                message: 'Tiles have been sorted Alphabetically.',
                backgroundColor: '#28a745',
                autoDismiss: true
            });
        } else {
            restoreOriginalTileOrder();
            new Toast({
                message: 'Tiles have been returned to Default sorting.',
                backgroundColor: '#28a745',
                autoDismiss: true
            });
        }
    }

    function sortTilesAlphabetically() {
        const tileContainer = document.querySelector('.tile-container');
        const tiles = Array.from(tileContainer.children);

        tiles.sort((a, b) => {
            const titleA = a.querySelector('.tile-text').textContent.toLowerCase();
            const titleB = b.querySelector('.tile-text').textContent.toLowerCase();
            return titleA.localeCompare(titleB);
        });

        tiles.forEach(tile => {
            tileContainer.appendChild(tile);
        });
    }

    function restoreOriginalTileOrder() {
        const tileContainer = document.querySelector('.tile-container');
        originalOrder.sort((a, b) => a.index - b.index).forEach(({ element }) => {
            tileContainer.appendChild(element);
        });
    }

    function addSheenToTile(hostId, sheenColor) {
        const tile = Array.from(document.querySelectorAll('.tile')).find(t => {
            return t.querySelector('.tile-text').textContent.trim().toLowerCase() === hostId.trim().toLowerCase();
        });

        if (tile) {
            tile.style.setProperty('--sheen-color', sheenColor);
            tile.classList.add('sheen');

            tile.addEventListener('click', () => {
                tile.classList.remove('sheen');
                tile.style.setProperty('--sheen-color', 'rgba(255, 255, 255, 0.5)');
            });
        } else {
            console.error(`Tile with hostId "${hostId}" not found.`);
        }
    }

    function toggleView() {
        const tileContainer = document.querySelector('.tile-container');
        
        if (tileContainer.classList.contains('list-view')) {
            switchToGridView();
        } else {
            switchToListView();
        }
    }    

    function switchToGridView() {
        const tileContainer = document.querySelector('.tile-container');
        const tiles = document.querySelectorAll('.tile');

        tileContainer.classList.remove('list-view');
        tiles.forEach(tile => {
            tile.classList.remove('list-view');
            tile.querySelector('.tile-data').classList.remove('list-view');
            tile.querySelector('.tile-icon').classList.remove('list-view');
        });

        document.getElementById('grid-view-button').classList.add('active');
        document.getElementById('list-view-button').classList.remove('active');

        localStorage.setItem('viewMode', 'grid');
    }

    function switchToListView() {
        const tileContainer = document.querySelector('.tile-container');
        const tiles = document.querySelectorAll('.tile');

        tileContainer.classList.add('list-view');
        tiles.forEach(tile => {
            tile.classList.add('list-view');
            tile.querySelector('.tile-data').classList.add('list-view');
            tile.querySelector('.tile-icon').classList.add('list-view');
        });

        document.getElementById('list-view-button').classList.add('active');
        document.getElementById('grid-view-button').classList.remove('active');

        localStorage.setItem('viewMode', 'list');
    }

    function applyTileColors() {
        // Define the primary and secondary colors
        const primaryColor = masterTileColor; // Primary color is the masterTileColor
        const secondaryColor = lightenDarkenColor(masterTileColor, -10); // Adjust this to get a suitable secondary color
    
        // Apply the colors to the CSS variables
        document.documentElement.style.setProperty('--primary-color', primaryColor);
        document.documentElement.style.setProperty('--secondary-color', secondaryColor);
    
        // Get all tile elements
        const tiles = document.querySelectorAll('.tile');
    
        // Loop through each tile and alternate colors
        tiles.forEach((tile, index) => {
            const hasCustomColor = tile.getAttribute('data-custom-color') === 'true';
            const hasOfflineColor = tile.getAttribute('data-offline-color') === 'true';

            // Only apply alternating colors if the tile doesn't have a custom color
            if (!hasCustomColor && !hasOfflineColor) {
                const tileColor = (index % 2 === 0) ? primaryColor : secondaryColor; // Alternate between primary and secondary
                tile.style.backgroundColor = tileColor; // Set the background color
            }
            if (hasCustomColor && !hasOfflineColor) {
                tile.style.backgroundColor = tile.color; // Set the background color
            }
        });
    }
    

    function lightenDarkenColor(color, percent) {
        let num = parseInt(color.slice(1), 16);
        let amt = Math.round(2.55 * percent);
        let R = (num >> 16) + amt;
        let G = (num >> 8 & 0x00FF) + amt;
        let B = (num & 0x0000FF) + amt;

        return (
            '#' +
            (
                0x1000000 +
                (R < 255 ? (R < 0 ? 0 : R) : 255) * 0x10000 +
                (G < 255 ? (G < 0 ? 0 : G) : 255) * 0x100 +
                (B < 255 ? (B < 0 ? 0 : B) : 255)
            )
                .toString(16)
                .slice(1)
        );
    }

    function closeSettingsPanel() {
        if (settingsPanel.classList.contains('visible')) {
            settingsPanel.classList.remove('visible');
            settingsPanel.classList.add('hidden');
        }
    }
    
    function createTileForHost(hostData) {
        const existingTile = tileMap.get(Number(hostData.id));
        
        if (existingTile) {
            existingTile.updateTileContent(hostData); // Update the existing tile if it already exists
        } else {
            let tempIndex = tileIndex++;

            let osName = hostData.os_name.toLowerCase();
            let osType = hostData.os_type.toLowerCase();
            let tempOSName = hostData.os_name;
            let osversion = hostData.os_version.includes(' (') ? hostData.os_version.split(' (')[0] : hostData.os_version;

            //macOS Wine Hack
            if (osName.includes('wine') || osType.includes('wine')) {
                osName = "macos";
                tempOSName = "macOS";
                osType = "darwin";
                osversion = "(Wine)";
            }
            
            //Catch Tumbleweed, Leap, etc.
            if (osName.includes('opensuse')) { osName = 'opensuse'; }

            //Debian Hack
            if (osName.includes('debian')) { osName = 'debian'; }

        
            const expectedIcon = `assets/images/server-${osName.replace(/\s+/g, '')}.png`;
            console.log(`${expectedIcon}`);
            const icon = assets.includes(expectedIcon)
                ? expectedIcon
                : `assets/images/server-${hostData.os_type.toLowerCase()}.png`;
        
        
            const newTile = new Tile({
                id: Number(hostData.id),
                name: hostData.hostname.length >= 18 ? hostData.hostname.substring(0, 18) : hostData.hostname,
                hostname: hostData.hostname,  // full hostname
                os_name: tempOSName,
                os_version: osversion,
                os_type: osType,
                main_ip: hostData.ip_address,
                ramUsage: hostData.memoryUsage,
                totalMemory: hostData.totalMemory,
                usedMemory: hostData.usedMemory,
                cpuUsage: hostData.cpuUsage,
                cpuCoreCount: hostData.cpuCoreCount,
                cpuClockSpeed: hostData.cpuClockSpeed,
                color: masterTileColor,
                icon: icon,
                data: ``,
                onClick: (tile) => {
                    tile.openHostDetailsWindow();
                }
            });

            newTile.element.dataset.index = tempIndex;
            originalOrder.push({ element: newTile.element, index: tempIndex });
            tileMap.set(hostData.id, newTile);  // Add new tile to the map
    
            // Append the new tile to the DOM
            const tileContainer = document.querySelector('.tile-container');
            if (tileContainer) {
                tileContainer.appendChild(newTile.element);
                setTimeout(() => {
                    newTile.element.classList.add('slide-in');
                }, 0);
            } else {
                console.error("Tile container not found");
            }
        }
    }
    
    function createTilesFromAPI(endpointUrl) {
        fetch(endpointUrl)
            .then(response => response.json())
            .then(data => {
                if (data.error) {
                    console.error("Error fetching data: ", data.message);
                    return;
                }
                
                const Hosts = data.data;
                        
                // Add/update tiles for new hosts
                Hosts.forEach(hostData => {
                    createTileForHost(hostData);
                });
    
        
                // Re-apply sorting and styles
                if (storedSortingEnabled) {
                    sortTilesAlphabetically();
                }
                if (storedStraightEdges) {
                    applyCornerStyle();
                }
                if (isRamSortingEnabled) {
                    sortTilesByRamUsage();
                }
                if (isCpuSortingEnabled) {
                    sortTilesByCpuUsage();
                }
                applyTileColors();
        
                // Set view mode
                const storedViewMode = localStorage.getItem('viewMode') || 'grid';
                if (storedViewMode === 'list') {
                    switchToListView();
                } else {
                    switchToGridView();
                }
            })
            .catch(error => {
                console.error("Error fetching data from API: ", error);
            });
    }